/*  libbadpenguin -- generic library for Bad Penguin development
 *
 *  Copyright (C) 1996 Antonio Gallo
 *  agx@linux.it
 *  
 *  Newer versions should be on www.badpenguin.org
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "badpenguin-chain.h"


// Allocate a new chain structure
TChain *chain_create() {
  return calloc( 1, sizeof(TChain) );
}


// Allocate a new chain item structure
TChainItem *chain_create_item() {
  return calloc( 1, sizeof(TChainItem) );
}


// Delete a whole chain structure but without disposing the pointer
int chain_cleanall(TChain *chain) {
  TChainItem *item, *old;
  if(chain==NULL){ 
    fprintf(stderr,"chain_cleanall: NULL pointer detected!\n");
    return 1;
  }
  // Delete Items
  item = chain->head;
  while (item!=NULL) {
    // Dispose data
    if (item->name != NULL) free(item->name);
    old = item;
    item = item->next;
    free(old);
  }
  // Reset chain
  chain->head = NULL;
  chain->tail = NULL;
  chain->count = 0;  
  return 0;
}


// Delete a whole chain structure and dispose also the pointer
int chain_delete(TChain *chain) {
  if(chain==NULL){ 
    fprintf(stderr,"chain_delete: NULL pointer detected!\n");
    return 1;
  }
  chain_cleanall(chain);
  free(chain);
  chain = NULL;
  return 0;
}


// Append a new item at the end of a chain
TChainItem *chain_append(TChain *chain) {
  if(chain==NULL){ 
    fprintf(stderr,"chain_append: NULL pointer detected!\n");
    return NULL;
  }
  // ... if the chain is empty
  if (chain->tail==NULL) {
    chain->head = chain_create_item();
    chain->head->chain= chain;    
    chain->tail = chain->head;
    chain->count= 1;
    return chain->tail;
  }
  // ... if the chain is not empty
  // append the new item
  chain->tail->next = chain_create_item();
  chain->tail->next->chain= chain;    
  chain->tail->next->prev = chain->tail;
  // move forward the tail
  chain->count++;
  chain->tail = chain->tail->next;
  return chain->tail;
}


// Delete an item from the chain
int chain_remove(TChainItem *item) {
  if(item==NULL){ 
    fprintf(stderr,"chain_remove: NULL item pointer detected!\n");
    return 1;
  }
  // ... if the prev is NULL then this is a Tail
  // i'm the first ?
  if (item->chain->head == item) item->chain->head = item->next;
  if (item->chain->tail == item) item->chain->tail = item->prev;
  if (item->prev != NULL) {
    item->prev->next = item->next;
  }
  if (item->next != NULL) {
    item->next->prev = item->prev;
  }
  (item->chain)->count --;
  // dispose
  free(item);
  return 0;
}


// Insert a new item after a chainItem with a specified name
TChainItem *chain_insert(TChain *chain, char *itemName, int order) {
  TChainItem *localItem, *newItem, *prevItem;
  int diff;
  
  if(chain==NULL){ 
    fprintf(stderr,"chain_insert: NULL pointer detected!\n");
    return NULL;
  }
  if(itemName==NULL){ 
    fprintf(stderr,"chain_insert: NULL item pointer detected!\n");
    return NULL;
  }

  localItem = chain->head;
  while (localItem != NULL ) {
    if (localItem->name != NULL ) {
      diff = strcmp(localItem->name,itemName);
      if ( ((order==1)&&(diff<0)) || ((order==0)&&(diff>0)) ) {
	// add new item
	newItem = chain_create_item();
	newItem->chain = chain;
	chain->count++;
	// set
	newItem->next = localItem;
        if (localItem->prev==NULL) {
	  chain->head = newItem;
	  localItem->prev = newItem;
	} else {
	  prevItem = localItem->prev;
	  newItem->prev = prevItem;
	  localItem->prev = newItem;
	  // adjust prev
	  prevItem->next = newItem;
	}	
	// done
	return newItem;
      }
    }
    localItem = localItem->next;
  }
  // hit bottom, i.e. not found maybe because the chain is empty,
  // appending the new item
  return chain_append(chain);
}

