/*  libbadpenguin -- generic library for Bad Penguin development
 *
 *  Copyright (C) 1996 Antonio Gallo
 *  agx@linux.it
 *  Written by Loren Lang, 9-26-01
 *  lorenl@tallye.com
 *  
 *  Newer versions should be on www.badpenguin.org
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 */

#include <config.h>

#include <sys/stat.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <libintl.h>
#define _(a)	dgettext(PACKAGE, a)

#include "badpenguin-ansi.h"
#include "badpenguin-errors.h"
#include "badpenguin-fs.h"

#define LINK_BUFFER_SIZE	128

int move_link( char* source, char* target) {
  int size = LINK_BUFFER_SIZE;
  int nchars;
  char *buffer = NULL;
  struct stat lpstat;
  
  // Try to rename
  if ( 0 == rename(source,target) ) return 0;
  
  // Move
  while(1) {
    buffer = realloc(buffer, size+1);
    if(buffer == NULL) {
      printERROR(_("move_link: could not allocate more memory!\n"));
      normal_color();
      return(-1);
    }
    if((nchars = readlink(source, buffer, size)) < size) {
	    free(buffer);
	    if(nchars < 0) {
	      printERROR(_("move_link: could not read symbolic link %s\n"), source);
	      normal_color();
	      return(-1);
	    }
	    buffer[nchars] = '\0';
	    if(symlink(buffer, target) < 0) {
	      printERROR(_("move_link: could not write symbolic link %s\n"), target);
	      normal_color();
	      return(-1);
	    }
	    lstat( source, &lpstat );
	    if ( 0 != lchown( target, lpstat.st_uid, lpstat.st_gid )) {
	      printERROR(_("move_link: unable to set UID and/or GID for %s\n"), target);
	      normal_color();        
	    }
	    if ( 0 != unlink(source) ) {
	      printERROR(_("move_link: unable to remove old file for %s\n"), target);
	      normal_color();        
	      return -1;
	    }
	    return(0);
    }
    size *= 2;
  }
}
